library(foreign)
library(nnet)
library(ggplot2)
library(reshape2)

#Multinomial logistic regression: Example with real data
#Entering high school students make program choices among general program, 
#vocational program and academic program. 
#Their choice might be modeled using their writing score and their social economic status.

data <- read.dta("https://stats.idre.ucla.edu/stat/data/hsbdemo.dta")

# The data set contains variables on 200 students. 
# The outcome variable (Y) is:
# "prog", program type. 
unique(data$prog)
# The predictor variables (X) are:
# "ses": social economic status, a three-level categorical variable 
unique(data$ses)
# "write": writing score, a continuous variable. 
hist(data$write)
min(data$write)
max(data$write)
#Descriptive statistics of the dataset.
summary(data)

#Descriptive statistics of the variables of interest: 
# Social economic Status vs Program type.
with(data=data, table(ses, prog))
# Writing score vs Program type.
with(data, do.call(rbind, tapply(write, prog, function(x) c(M = mean(x), SD = sd(x)))))

# The levels of a factor are re-ordered so that the level specified by ref 
# is first and the others are moved down. 
data$prog2 <- relevel(data$prog, ref = "academic")
# "acamedic" is now the baseline category
model <- multinom(prog2 ~ ses + write, data = data)
summary(model)


z <- summary(model)$coefficients/summary(model)$standard.errors
z

# 2-tailed z test
p <- (1 - pnorm(abs(z), 0, 1)) * 2
p


#We first see that some output is generated by running the model, even though we are 
#assigning the model to a new R object. This model-running output includes some iteration
#history and includes the final negative log-likelihood 179.981726. This value multiplied 
#by two is then seen in the model summary as the Residual Deviance and it can be used in 
#comparisons of nested models, but we won’t show an example of comparing models on this page.
#The model summary output has a block of coefficients and a block of standard errors. 
#Each of these blocks has one row of values corresponding to a model equation. 
#Focusing on the block of coefficients, we can look at the first row comparing 
#prog = "general" to our baseline prog = "academic" and the second row comparing 
#prog = "vocation" to our baseline prog = "academic". If we consider our coefficients 
#from the first row to be b1 and our coefficients from the second row to be b2, we can 
#write our model equations:






# Effect of variable "write" on the response:
# b13 A one-unit increase in the variable write is associated with the decrease in the 
# log odds of being in general program vs. academic program in the amount of .058 .
# b23 A one-unit increase in the variable write is associated with the decrease in the
# log odds of being in vocation program vs. academic program. in the amount of .1136 


# Effect of variable "socio economic status" (ses) on the response:
#.b12 The log odds of being in general program vs in academic program will decrease 
#by 1.163 if moving from ses="low" to ses="high"
#.b11 The log odds of being in general program vs. in academic program will decrease 
#by 0.533 if moving from ses="low"to ses="middle", although this coefficient is not significant
#.b22 The log odds of being in vocation program vs. in academic program will decrease
#by 0.983 if moving from ses="low" to ses="high"
#.b21 The log odds of being in vocation program vs. in academic program will increase
#by 0.291 if moving from ses="low" to ses="middle", although this coefficient is not significant.


#The ratio of the probability of choosing one outcome category over the probability of 
#choosing the baseline category is often referred as relative risk (and it is sometimes
#referred to as odds, described in the regression parameters above). 
#The relative risk is the right-hand side linear equation exponentiated, leading to the fact
#that the exponentiated regression coefficients are relative risk ratios for a unit change 
#in the predictor variable. We can exponentiate the coefficients from our model to see these
#risk ratios.

exp(coef(model))

# The relative risk ratio for a one-unit increase in the variable write is .9437 for being 
#in general program vs. academic program.

#The relative risk ratio switching from ses = 1 to 3 is .3126 for being in general 
#program vs. academic program.

# You can also use predicted probabilities to help you understand the model. 
# You can calculate predicted probabilities for each of our outcome levels using the fitted function. 
# We can start by generating the predicted probabilities for the observations in our dataset 
# and viewing the first few rows

head(pp <- fitted(model))

# Next, if we want to examine the changes in predicted probability associated with one of our 
# two variables, we can create small datasets varying one variable while holding the other constant. 
# We will first do this holding write at its mean and examining the predicted probabilities
# for each level of ses.

dses <- data.frame(ses = c("low", "middle", "high"), write = mean(ml$write))
predict(model, newdata = dses, "probs")


#Averaged predicted probabilities for different values of the continuous predictor variable 
#write within each level of ses.
dwrite <- data.frame(ses = rep(c("low", "middle", "high"), each = 41), write = rep(c(30:70),
                                                                                   3))

## store the predicted probabilities for each value of ses and write
pp.write <- cbind(dwrite, predict(model, newdata = dwrite, type = "probs", se = TRUE))

## calculate the mean probabilities within each level of ses
by(pp.write[, 3:5], pp.write$ses, colMeans)

## melt data set to long for ggplot2
lpp <- melt(pp.write, id.vars = c("ses", "write"), value.name = "probability")
head(lpp)  # view first few rows

## plot predicted probabilities across write values for each level of ses
## facetted by program type
ggplot(lpp, aes(x = write, y = probability, colour = ses)) + geom_line() + facet_grid(variable ~
                                                                                        ., scales = "free")